/*

 Author: 
  Sebastien Verel, 
  Univ. du Littoral Côte d'Opale, France.
  
*/

#ifndef _adadeltaGradientDescent_h
#define _adadeltaGradientDescent_h

#include <vector>
#include <math.h>

#include <gradientDescent.h>
#include <lossFunction.h>

class AdaDeltaGradientDescent : public GradientDescent {
public:
    AdaDeltaGradientDescent(LossFunction & _loss, double _gamma, double _epsilon, unsigned _nIter) : loss(_loss), gamma(_gamma), epsilon(_epsilon), nIter(_nIter) {

    }

    virtual void optim(std::vector<double> & theta) {
        std::vector<double> grad(theta.size());
        std::vector<double> delta(theta.size());

        std::vector<double> G(theta.size(), 0);
        std::vector<double> Edelta(theta.size(), 0);

        for(unsigned iter = 0; iter < nIter; iter++) {
            loss.gradient(theta, grad);

            for(unsigned k = 0; k < theta.size(); k++) {
                G[k] = gamma * G[k] + (1 - gamma) * grad[k] * grad[k];
                
                delta[k] = - sqrt(Edelta[k] + epsilon) / sqrt(G[k] + epsilon) * grad[k];
    
                // x = x + deltaTheta
                theta[k] += delta[k];

                Edelta[k] = gamma * Edelta[k] + (1 - gamma) * delta[k] * delta[k];
            }

        }
    }

protected:
    LossFunction & loss;

    double gamma;
    double epsilon;

    unsigned nIter;

};

#endif
