/*

 Author: 
  Sebastien Verel, 
  Univ. du Littoral Côte d'Opale, France.
  
*/

#ifndef _adaGradientDescent_h
#define _adaGradientDescent_h

#include <vector>
#include <math.h>

#include <gradientDescent.h>
#include <lossFunction.h>

class AdaGradientDescent : public GradientDescent {
public:
    AdaGradientDescent(LossFunction & _loss, double _sigma, double _epsilon, unsigned _nIter) : loss(_loss), sigma(_sigma), epsilon(_epsilon), nIter(_nIter) {

    }

    virtual void optim(std::vector<double> & x) {
        std::vector<double> grad(x.size());

        std::vector<double> G(x.size(), 0);

        for(unsigned iter = 0; iter < nIter; iter++) {
            loss.gradient(x, grad);

            for(unsigned k = 0; k < x.size(); k++) {
                G[k] += grad[k] * grad[k];
            }

            // x = x - sigma * grad f
            for(unsigned k = 0; k < x.size(); k++) {
                x[k] -= sigma / (sqrt(G[k] + epsilon)) * grad[k];
            }

        }
    }

protected:
    LossFunction & loss;

    double sigma;
    double epsilon;

    unsigned nIter;

};

#endif
