/*

 Author: 
  Sebastien Verel, 
  Univ. du Littoral Côte d'Opale, France.
  
*/

#ifndef _nadamGradientDescent_h
#define _nadamGradientDescent_h

#include <vector>

#include <gradientDescent.h>
#include <lossFunction.h>

class NadamGradientDescent : public GradientDescent {
public:
    NadamGradientDescent(LossFunction & _loss, double _sigma, double _beta1, double _beta2, unsigned _nIter) : loss(_loss), sigma(_sigma), beta1(_beta1), beta2(_beta2), nIter(_nIter) {
        epsilon = 1e-8;
    }

    virtual void optim(std::vector<double> & theta) {
        std::vector<double> grad(theta.size());
        std::vector<double> m(theta.size(), 0);
        std::vector<double> v(theta.size(), 0);

        double beta1t = beta1;
        double beta2t = beta2;
        double hatm, hatv;
        for(unsigned iter = 0; iter < nIter; iter++) {
            loss.gradient(theta, grad);

            for(unsigned k = 0; k < v.size(); k++) {
                m[k] = beta1 * m[k] + (1 - beta1) * grad[k];
                v[k] = beta2 * v[k] + (1 - beta2) * grad[k] * grad[k];

                // theta = theta - sigma / (sqrt(hat(v) + epsilon) * ( beta * hat(m) + (1-beta)/(1-beta1^t) * grad f)
                hatm = m[k] / (1 - beta1t);
                hatv = v[k] / (1 - beta2t);
                theta[k] -= sigma / (sqrt(hatv) + epsilon) * (beta1 * hatm + (1 - beta1) / (1 - beta1t) * grad[k]) ;
            }

            beta1t *= beta1;
            beta2t *= beta2;
        }
    }

protected:
    LossFunction & loss;

    double epsilon;
    double sigma;
    double beta1, beta2;

    unsigned nIter;

};

#endif
