/*

 Author: 
  Sebastien Verel, 
  Univ. du Littoral Côte d'Opale, France.
  
*/

#ifndef _nesterovGradientDescent_h
#define _nesterovGradientDescent_h

#include <vector>

#include <gradientDescent.h>
#include <lossFunction.h>

class NesterovGradientDescent : public GradientDescent {
public:
    NesterovGradientDescent(LossFunction & _loss, double _sigma, double _gamma, unsigned _nIter) : loss(_loss), sigma(_sigma), gamma(_gamma), nIter(_nIter) {

    }

    virtual void optim(std::vector<double> & x) {
        std::vector<double> grad(x.size());
        std::vector<double> v(x.size());
        std::vector<double> xp(x.size());

        unsigned k;
        for(unsigned iter = 0; iter < nIter; iter++) {
            for(k = 0; k < x.size(); k++) {
                xp[k] = x[k] - sigma * v[k];
            }

            loss.gradient(xp, grad);

            for(k = 0; k < v.size(); k++) {
                v[k] = gamma * v[k] + sigma * grad[k];
            }

            // x = x - sigma * grad f
            for(k = 0; k < x.size(); k++) {
                x[k] -= v[k];
            }

        }
    }

protected:
    LossFunction & loss;

    double sigma;
    double gamma;

    unsigned nIter;

};

#endif
