/*

 Author: 
  Sebastien Verel, 
  Univ. du Littoral Côte d'Opale, France.
  
*/

#ifndef _stochasticGradientDescent_h
#define _stochasticGradientDescent_h

#include <vector>
#include <algorithm>
#include <random>

#include <lossFunction.h>
#include <gradientDescent.h>

class StochasticGradientDescent : public GradientDescent {
public:
    StochasticGradientDescent(std::mt19937 & _gen, LossFunction & _loss, double _sigma, unsigned _nIter) : gen(_gen), loss(_loss), sigma(_sigma), nIter(_nIter) {

    }

    virtual void optim(std::vector<double> & x) {
        std::vector<double> grad(x.size());

        // index of the data
        std::vector<unsigned> index(loss.data.x.size());
        for(unsigned i = 0; i < index.size(); i++)
            index[i] = i;

        for(unsigned iter = 0; iter < nIter; iter++) {
            // shuffle the index
            std::shuffle(std::begin(index), std::end(index), gen);

            for(unsigned i = 0; i < loss.data.x.size(); i++) {
                loss.gradient(x, index[i], grad);                

                // x = x - sigma * grad(i) f
                for(unsigned k = 0; k < x.size(); k++) {
                    x[k] -= sigma * grad[k];
                }
            }
        }
    }

protected:
    std::mt19937 & gen;

    LossFunction & loss;

    double sigma;

    unsigned nIter;

};

#endif
